/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Contributor(s): Billy <bllaw@mmu.edu.my> http://linkvisitor.mozdev.org/
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var LinkVisitor = {
  _globalHistory : Components.classes['@mozilla.org/browser/global-history;2'].getService(Components.interfaces.nsIGlobalHistory2),
  _browserHistory : Components.classes["@mozilla.org/browser/global-history;2"].getService(Components.interfaces.nsIBrowserHistory),
  _URIFixup : Components.classes["@mozilla.org/docshell/urifixup;1"].getService(Components.interfaces.nsIURIFixup),
  _bookmarksService : Components.classes['@mozilla.org/browser/bookmarks-service;1'].getService(Components.interfaces.nsIBookmarksService),
  _pref : Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(null),

  // This is the node user right clicks on, null if it is not an address node.
  // It is initiliased and reassigned in _isOverLink() only.
  _overLink : null,

  // Modified from optimoz mosgestoverlay.js
  // Given a node, try to get the address node of itself or its parent(s).
  // Returns null if cannot find any.
  _findNodeLink : function(domPosition) {
    if (domPosition == document)
      return null;
    try {
      if(domPosition.nodeName == "A")
        return {node : domPosition, uri : domPosition.href};
      else
        return this._findNodeLink(domPosition.parentNode);
    }
    catch(e) {
      return null;
    }
  },

  // Returns true if the popup node is an address node.
  _isOverLink : function() {
    this._overLink = this._findNodeLink(document.popupNode);
    return (this._overLink != null);
  },

  // Idea taken from Tabbrowser Extensions globaloverlay.js
  // Once we added/removed the url to the history list,
  // doing this will change the address node's colour.
  // Unfortunately, it will only affect the node, and not other
  // nodes with the same url in the page.
  _refreshLinkColor : function(aNode) {
    var oldHref = aNode.href;
    aNode.href = '';
    aNode.href = oldHref;
  },

  // This function is invoked in window (?) context,
  // so use 'LinkVisitor' instead of 'this'.
  // event.originalTarget is the loaded document.
  onWindowLoad : function(event) {
    var parentPopup = document.getElementById('contentAreaContextMenu');
    parentPopup.addEventListener("popupshown", LinkVisitor._onParentPopupShown, true);

    if (LinkVisitor._isBookmarkShownVisited())
      LinkVisitor._markBookmarkedLinksAsVisited(event.originalTarget);
  },

  // This function is invoked in window (?) context,
  // so use 'LinkVisitor' instead of 'this'
  _onParentPopupShown : function() {
    if (LinkVisitor._isOverLink()) {
      document.getElementById('vlToggleLink').setAttribute('hidden', false);

      var fixupURI = LinkVisitor._getFixupURI(LinkVisitor._overLink.uri);
      var visited = LinkVisitor._globalHistory.isVisited(fixupURI);
      var bundle = document.getElementById("vlStringBundle");
      if (visited)
        document.getElementById('vlToggleLink').label = bundle.getString('vlToggleLinkVisited');
      else
        document.getElementById('vlToggleLink').label = bundle.getString('vlToggleLinkUnvisited');
    }
    else
      document.getElementById('vlToggleLink').setAttribute('hidden', true);
  },

  // If this function is invoked from the context menu, 
  // onParentPopupShown will always be invoked first.
  onPopupShown : function() {
    var selectedText = String(this._getSelection());
    var textSelected = selectedText.length > 0;

    var hasLink =
      ("links" in document.commandDispatcher.focusedWindow.document) &&
      (document.commandDispatcher.focusedWindow.document.links.length > 0);
    var linkSelected = textSelected && hasLink;

    document.getElementById('vlVisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlUnvisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlSelVisitAll').setAttribute('disabled', !linkSelected);
    document.getElementById('vlSelUnvisitAll').setAttribute('disabled', !linkSelected);

    document.getElementById('vlMVisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlMUnvisitAll').setAttribute('disabled', !hasLink);
    document.getElementById('vlMSelVisitAll').setAttribute('disabled', !linkSelected);
    document.getElementById('vlMSelUnvisitAll').setAttribute('disabled', !linkSelected);

    var bookmarkVisited = this._isBookmarkShownVisited();
    document.getElementById('vlBookmarkVisited').setAttribute('checked', bookmarkVisited);
    document.getElementById('vlMBookmarkVisited').setAttribute('checked', bookmarkVisited);
  },

  // Modified from linky linky.js
  _getSelection : function() {
    var focusedWindow = document.commandDispatcher.focusedWindow;
    return focusedWindow.__proto__.getSelection.call(focusedWindow);
  },

  // Do the users want bookmarks always shown visited?
  _isBookmarkShownVisited : function() {
    try {
      return this._pref.getBoolPref('extensions.linkvisitor.always_mark_bookmarks_visited');
    }  
    catch(e) {
      return false;
    }
  },

  // Returns true if the url is bookmarked.
  // It seems that nsiBookmarksService uses nsiFixupURI to
  // fix the url, but to be safe, call _getFixupURI and use fixupURI.spec to
  // fix the url ourselves first before calling this function.
  _isBookmarked : function(url) {
    try {
      var found = this._bookmarksService.isBookmarked(url);
      return found;
    }
    catch(e) {
      return false;
    }
  },

  toggleLink : function() {
    if (this._overLink == null)
      return;

    var fixupURI = this._getFixupURI(this._overLink.uri);
    var visited = this._globalHistory.isVisited(fixupURI);
    //  alert('visited: ' + visited);
    if (visited) {
      if (this._isBookmarkShownVisited() && this._isBookmarked(fixupURI.spec))
        alert(document.getElementById('vlStringBundle').getString('vlBookmarkVisited'));
      else
        this._browserHistory.removePage(fixupURI.spec);
    }
    else
      this._globalHistory.addURI(fixupURI, false, true);

    this._refreshLinkColor(this._overLink.node);
  },

  _getFixupURI : function(uri) {
    try {
      var fixedURI = this._URIFixup.createFixupURI(uri, 0);
      return fixedURI;
    }
    catch(e) {
      return null;
    }
  },

  // Modified from linky linky.js
  markSelectedLinks : function(wantVisited) {
    var selection = this._getSelection();

    if (String(selection).length == 0 ||
        !("links" in document.commandDispatcher.focusedWindow.document))
      return;

    var links = document.commandDispatcher.focusedWindow.document.links;
    if (links.length == 0)
      return;

    var bookmarkVisited = this._isBookmarkShownVisited();
    var bookmarkCount = 0;

    // not very efficient, but it does the work all right.
    for (var i = 0; i < links.length; i++) {
      if (selection.containsNode(links[i], true)) {
        var fixupURI = this._getFixupURI(links[i].href);
        if (wantVisited)
          this._globalHistory.addURI(fixupURI, false, true);
        else {
          if (bookmarkVisited && this._isBookmarked(fixupURI.spec) && this._globalHistory.isVisited(fixupURI))
            bookmarkCount++; // do not remove the link from history
          else
            this._browserHistory.removePage(fixupURI.spec);
        }
        this._refreshLinkColor(links[i]);
      }
    }

    if (bookmarkVisited && bookmarkCount > 0) {
      var text = document.getElementById('vlStringBundle').getString('vlBookmarkCount');
      text = this._replaceInsert(text, 1, bookmarkCount);
      alert(text);
    }
  },

  markAllLinks : function(wantVisited) {
    if (!("links" in document.commandDispatcher.focusedWindow.document))
      return;

    var links = document.commandDispatcher.focusedWindow.document.links;
    if (links.length == 0)
      return;

    var bookmarkVisited = this._isBookmarkShownVisited();
    var bookmarkCount = 0;

    for (var i = 0; i < links.length; i++) {
      var fixupURI = this._getFixupURI(links[i].href);
      if (wantVisited)
        this._globalHistory.addURI(fixupURI, false, true);
      else {
        if (bookmarkVisited && this._isBookmarked(fixupURI.spec) && this._globalHistory.isVisited(fixupURI))
          bookmarkCount++; // do not remove the link from history
        else
          this._browserHistory.removePage(fixupURI.spec);
      }
      this._refreshLinkColor(links[i]);
    }

    if (bookmarkVisited && bookmarkCount > 0) {
      var text = document.getElementById('vlStringBundle').getString('vlBookmarkCount');
      text = this._replaceInsert(text, 1, bookmarkCount);
      alert(text);
    }
  },

  toggleMarkBookmarksVisited : function() {
    var enabled = !this._isBookmarkShownVisited();
    try {
      this._pref.setBoolPref('extensions.linkvisitor.always_mark_bookmarks_visited', enabled);
    }  
    catch(e) {}

    if (enabled)
      this._markBookmarkedLinksAsVisited(document.commandDispatcher.focusedWindow.document);
  },

  // Traverse all links in document, and add the links to
  // history if they are not visited but bookmarked.
  _markBookmarkedLinksAsVisited : function(doc) {
    if (!("links" in doc))
      return;

    var links = doc.links;
    if (links.length == 0)
      return;

    for (var i = 0; i < links.length; i++) {
      var fixupURI = this._getFixupURI(links[i].href);
      if (!this._globalHistory.isVisited(fixupURI) && this._isBookmarked(fixupURI.spec)) {
        this._globalHistory.addURI(fixupURI, false, true);
        this._refreshLinkColor(links[i]);
      }
    }
  },

  // Taken from linky's shared.js
  _replaceInsert : function(text, index, value) {
    var result = text;
    var re = new RegExp("#" + index);
    result = result.replace(re, value);
    return result;
  },
};

// Open the History Window in Mozilla and History Sidebar in Mozilla Firebird
function lvShowHistoryList() {
  try {
    toHistory(); // open the History Window in Mozilla
  }
  catch(e) {
    toggleSidebar("viewHistorySidebar"); // should be in Mozilla Firebird
  }
}

window.addEventListener("load", LinkVisitor.onWindowLoad, true);
